@@ -3,7 +3,7 @@ dwiRiceBias <- function(object,  ...) cat("No Rice Bias correction defined for t
 setGeneric("dwiRiceBias", function(object,  ...) standardGeneric("dwiRiceBias"))
 
 setMethod("dwiRiceBias","dtiData",function(object, 
-sigma=NULL, method="1stMoment") {
+sigma=NULL, ncoils=1) {
 args <- object@call
 if(is.null(sigma)||sigma<1){
 cat("please provide a value for sigma \n Returning the original object\n")
@@ -19,7 +19,7 @@ cat("\n Returning the original object\n")
 }
 }
 if(!corrected){
-object@si <- array(ricebiascorr(object@si,sigma),dim(object@si))
+object@si <- array(ricebiascorr(object@si,sigma, ncoils),dim(object@si))
 object@call <- c(args,sys.call(-1))
 }
 invisible(object)
@@ -33,7 +33,10 @@ x2 <- rnorm(x,0,s)
 sqrt(x1*x1+x2*x2)
 }
 
-ricebiascorr <- function(x,s=1,method=1){
+ricebiascorr0 <- function(x,s=1,ncoils=1){
+xt <- x/s
+if(ncoils==1){
+# define functions needed
 Lhalf <- function(x){
 (1-x)*besselI(-x/2,0,TRUE) - x*besselI(-x/2,1,TRUE)
 }
@@ -42,19 +45,42 @@ fofx <- function(x) {
       x[ind] <- sqrt(pi/2)*Lhalf(-x[ind]^2/2)
       x
 }
-xt <- x/s
-if(method==1){
 xt <- .Fortran("rcstep",xt=as.double(xt),
                as.integer(length(xt)),
                DUPL=TRUE,
                PACKAGE="dti")$xt
 } else {
-y <- xt
-xt <- sqrt(pmax(0,y^2-2))
+# define functions needed
+m1chiL <- function(L,eta){
+require(gsl)
+sqrt(pi/2)*poch(L,.5)/factorial(.5)*hyperg_1F1(-.5,L,-eta^2/2)
+}
+etasolve <- function(L,m1,eps=.001){
+maxeta <- max(m1+1e-10)
+x <- seq(0,maxeta,eps)
+lx <- length(x)
+fx <- m1chiL(L,x)
+m1s[m1s<min(fx)] <- min(fx)
+ind <- cut(m1+1e-10, breaks = fx, labels=FALSE)
+ind1 <- pmin(ind+1,length(fx)-1)
+z <- (x[ind1]*(m1s-fx[ind])+ x[ind]*(fx[ind1]-m1s))/(fx[ind1]-fx[ind])
+z[is.na(z)] <- m1[is.na(z)]
+z
+}
+xt[xt<100] <- etasolve(ncoils,xt[xt<100])
 }
 xt*s
 }
 
+ricebiascorr <- function(x,s=1,ncoils=1){
+varstats <- sofmchi(ncoils,50,.002)
+xt <- x/s
+xt <- pmax(varstats$minlev,xt)
+ind <- 
+findInterval(xt, varstats$mu, rightmost.closed = FALSE, all.inside = FALSE)
+varstats$ncp[ind]*s
+}
+
 betaL <- function(L){
 # L-number of coils, eta - noncentrality parameter of\chi_L
 require(gsl)
@@ -90,7 +116,7 @@ eta[ind] <- etanew
 }
 eta
 }
-etasolve <- function(L,m1,sigma,eps=.01){
+etasolve <- function(L,m1,sigma,eps=.001){
 m1s <- m1/sigma
 maxeta <- max(m1s+1e-10)
 x <- seq(0,maxeta,eps)
@@ -99,5 +125,7 @@ fx <- m1chiL(L,x)
 m1s[m1s<min(fx)] <- min(fx)
 ind <- cut(m1s+1e-10, breaks = fx, labels=FALSE)
 ind1 <- pmin(ind+1,length(fx)-1)
-(x[ind1]*(m1s-fx[ind])+ x[ind]*(fx[ind1]-m1s))/(fx[ind1]-fx[ind])
+z <- (x[ind1]*(m1s-fx[ind])+ x[ind]*(fx[ind1]-m1s))/(fx[ind1]-fx[ind])
+z[is.na(z)] <- m1[is.na(z)]
+z
 }
