@@ -4,6 +4,166 @@
 #
 #
 awssigmc <- function(y,                 # data
+                     steps,             # number of iteration steps for PS
+                     mask = NULL,       # data mask, where to do estimation
+                     ncoils = 1,        # number of coils for parallel MR image acquisition
+                     vext = c( 1, 1),   # voxel extensions
+                     lambda = 20,       # adaptation parameter for PS
+                     h0 = 2,            # initial bandwidth for first step in PS
+                     verbose = FALSE, 
+                     sequence = FALSE,  # return estimated sigma for intermediate steps of PS?
+                     hadj = 1,          # adjust parameter for density() call for mode estimation
+                     q = .25,  # for IQR
+                     qni = .8,
+                     method=c("VAR","MAD")  # for variance, alternative "MAD" for mean absolute deviation
+                     ) {
+  method <- match.arg(method)
+  ## some functions for pilot estimates
+  IQQ <- function (x, q = .25, na.rm = FALSE, type = 7) 
+    diff(quantile(as.numeric(x), c(q, 1-q), na.rm = na.rm, names = FALSE, type = type))
+
+  IQQdiff <- function(y, mask, q = .25, verbose = FALSE) {
+    cq <- qnorm(1-q)*sqrt(2)*2
+    sx <- IQQ( diff(y[mask]), q)/cq
+    sy <- IQQ( diff(aperm(y,c(2,1,3))[aperm(mask,c(2,1,3))]), q)/cq
+    sz <- IQQ( diff(aperm(y,c(3,1,2))[aperm(mask,c(3,1,2))]), q)/cq
+    if(verbose) cat( "Pilot estimates of sigma", sx, sy, sz, "\n")
+    min( sx, sy, sz)
+  }
+  
+  estsigma <- function(y, mask, q, L, sigma, verbose=FALSE){
+    meany <- mean(y[mask]^2)
+    eta <- sqrt(max( 0, meany/sigma^2-2*L))
+    m <- sqrt(pi/2)*gamma(L+1/2)/gamma(L)/gamma(3/2)*hyperg_1F1(-1/2,L,-eta^2/2)
+    v <- max( .01, 2*L+eta^2-m^2)
+    if(verbose) cat( eta, m, v, "\n")
+    IQQdiff( y, mask, q)/sqrt(v)
+  }
+  
+  varstats <- sofmchi(ncoils)
+  if(length(vext)==3) vext <- vext[2:3]/vext[1]
+  ## dimension and size of cubus
+  ddim <- dim(y)
+  n <- prod(ddim)
+
+  ## test dimension
+  if (length(ddim) != 3) stop("first argument should be a 3-dimentional array")
+
+  ## check mask
+  if (is.null(mask)) mask <- array(TRUE, ddim)
+  if(length(mask) != n) stop("dimensions of data array and mask should coincide")
+
+  ## initial value for sigma_0 
+  # sigma <- sqrt( mean( y[mask]^2) / 2 / ncoils)
+  sigma <- IQQdiff( y, mask, q, verbose=verbose)
+#  cat( "sigmahat1", sigma, "\n")
+  sigma <- estsigma( y, mask, q, ncoils, sigma)
+#  cat( "sigmahat2", sigma, "\n")
+  sigma <- estsigma( y, mask, q, ncoils, sigma)
+#  cat( "sigmahat3", sigma, "\n")
+  sigma <- estsigma( y, mask, q, ncoils, sigma, verbose=verbose)
+#  cat( "sigmahat4", sigma,"\n")
+
+  ## define initial arrays for parameter estimates and sum of weights (see PS)
+  th <- array( 1, ddim)
+  ni <- array( 1, ddim)
+#  y <- y/sigma # rescale to avoid passing sigma to awsvchi
+  minlev <- sqrt(2)*gamma(ncoils+.5)/gamma(ncoils)
+  if (sequence) sigmas <- lind <- minni <- numeric(steps)
+  mc.cores <- setCores(,reprt=FALSE)
+  ## iterate PS starting with bandwidth h0
+  for (i in 1:steps) {
+
+    h <- h0 * 1.25^((i-1)/3)
+    nw <- prod(2*as.integer(h/c(1,vext))+1)
+#    cat("nw=",nw,"h/vext=",h/c(1,1/vext),"ih=",as.integer(h/c(1,1/vext)),"\n")
+    param <- .Fortran("paramw3",
+                      as.double(h),
+                      as.double(vext),
+                      ind=integer(3*nw),
+                      w=double(nw),
+                      n=as.integer(nw),
+                      DUPL = FALSE,
+                      PACKAGE = "dti")[c("ind","w","n")]
+    nw <- param$n
+    param$ind <- param$ind[1:(3*nw)]
+    dim(param$ind) <- c(3,nw)
+    param$w   <- param$w[1:nw]    
+    fncchi <- fncchiv(th/sigma,varstats)
+## correction factor for variance of NC Chi distribution
+    ## perform one step PS with bandwidth h
+    if(method=="VAR"){
+    z <- .Fortran("awsvchi",
+                  as.double(y),        # data
+                  as.double(th),       # previous estimates
+                  ni = as.double(ni/fncchi),
+                  as.logical(mask),
+                  as.integer(ddim[1]),
+                  as.integer(ddim[2]),
+                  as.integer(ddim[3]),
+                  as.integer(param$ind),
+                  as.double(param$w),
+                  as.integer(nw),
+                  as.double(lambda),
+                  as.double(sigma),
+                  th = double(n),
+                  sy = double(n),
+                  DUPL = FALSE,
+                  PACKAGE = "dti")[c("ni","th","sy")]
+    } else {
+    z <- .Fortran("awsadchi",
+                  as.double(y),        # data
+                  as.double(th),       # previous estimates
+                  ni = as.double(ni/fncchi),
+                  as.logical(mask),
+                  as.integer(ddim[1]),
+                  as.integer(ddim[2]),
+                  as.integer(ddim[3]),
+                  as.integer(param$ind),
+                  as.double(param$w),
+                  as.integer(nw),
+                  as.double(lambda),
+                  as.double(sigma),
+                  double(nw*mc.cores),
+                  as.integer(mc.cores),
+                  th = double(n),
+                  sy = double(n),
+                  DUPL = FALSE,
+                  PACKAGE = "dti")[c("ni","th","sy")]       
+    }
+    ## extract sum of weigths (see PS) and consider only voxels with ni larger then mean
+    th <- z$th
+    ni <- z$ni
+    ni[!mask]<-1
+    ind <- (ni > .9999*quantile(ni[ni>1],qni))#&(z$th>sigma*minlev)
+## use correction factor for sd of NC Chi distribution
+    sy <- z$sy[ind]/fncchis(th[ind]/sigma,varstats)
+    ## use the maximal mode of estimated local sd parameters, exclude largest values for better precision
+    dsigma <- density( sy[sy>0], n = 4092, adjust = hadj, to = min( max(sy[sy>0]), median(sy[sy>0])*5) )
+    sigma <- dsigma$x[dsigma$y == max(dsigma$y)][1]
+
+    if (sequence) {
+         sigmas[i] <- sigma
+         lind[i] <- sum(ind)
+         minni[i] <- min(ni[ind])
+         }
+
+    if (verbose) {
+      plot(dsigma, main = paste( "estimated sigmas step", i, "h=", signif(h,3)))
+      cat( "step", i, "h=", signif( h, 3), "quantiles of ni", signif( quantile(ni[ind]), 3), "mean", signif( mean(ni[ind]), 3), "\n")
+      cat( "quantiles of sigma", signif( quantile(sy[sy>0]), 3), "mode", signif( sigma, 3), "\n")
+    }
+  }
+  ## END PS iteration
+
+
+  ## this is the result (th is expectation, not the non-centrality parameter !!!)
+  invisible(list(sigma = if(sequence) sigmas else sigma,
+                 theta = th, 
+                 lind  = if(sequence) lind else sum(ind), 
+                 minni  = if(sequence) minni else min(ni[ind])))
+}
+awssigmc0 <- function(y,                 # data
                      steps,             # number of iteration steps for PS
                      mask = NULL,       # data mask, where to do estimation
                      ncoils = 1,        # number of coils for parallel MR image acquisition
@@ -11,7 +171,6 @@ awssigmc <- function(y,                 # data
                      lambda = 10,       # adaptation parameter for PS
                      h0 = 2,            # initial bandwidth for first step in PS
                      verbose = FALSE, 
-                     model = "chisq",   # use chi or chisq data (and corr. moments)
                      sequence = FALSE,  # return estimated sigma for intermediate steps of PS?
                      eps = 1e-5,        # accuracy for fixpoint iteration 
                      hadj = 1,          # adjust parameter for density() call for mode estimation
@@ -65,10 +224,10 @@ awssigmc <- function(y,                 # data
   ## define initial arrays for parameter estimates and sum of weights (see PS)
   th <- array( 1, ddim)
   ni <- array( 1, ddim)
-
+  y <- y/sigma # rescale to avoid passing sigma to awsvchi2
   ## use squared quantities for chisq method
-  if (model == "chisq") y <- y^2
-
+  y <- y^2
+    
   if (sequence) sigmas <- numeric(steps)
 
   ## iterate PS starting with bandwidth h0
@@ -91,7 +250,7 @@ awssigmc <- function(y,                 # data
                   th2 = double(n),
                   ni2 = double(n),
                   double(n),           # array to precompute lgamma
-                  as.double(sigma^2),
+#                  as.double(sigma^2),
                   as.double(h),
                   as.double(vext),
                   DUPL = FALSE,
@@ -102,8 +261,6 @@ awssigmc <- function(y,                 # data
     ind <- ni > mean(ni)
     cw <- z$ni2[ind]/ni[ind]^2
 
-    if (model == "chisq") {
-      
       th <- z$th
       m1 <- th[ind]
       mu <- pmax( 1/(1-cw)*(z$th2[ind]-m1^2), 0)
@@ -111,20 +268,12 @@ awssigmc <- function(y,                 # data
       indt <- m1^2 - mu*ncoils < 0
       s2 <- sqrt((m1-sqrt(pmax( 0, m1^2-mu*ncoils)))/p)
 
-    } else { # fixpoint equation
-
-      th <- z$th2
-      m1 <- z$th[ind]
-      indt <- th[ind]-m1^2<0
-      mu <- pmax(1/(1-cw)*(th[ind]-m1^2),0)
-      eta <- fixpetaL(ncoils, rep(1,sum(ind)), m1, mu, eps = eps, maxcount = 500)
-      s2 <- (m1/m1chiL( ncoils, eta))
-
-    }
-
     ## use the maximal mode of estimated local variance parameters, exclude largest values for better precision
     dsigma <- density( s2[s2>0], n = 4092, adjust = hadj, to = min( max(s2[s2>0]), median(s2[s2>0])*5) )
-    sigma <- dsigma$x[dsigma$y == max(dsigma$y)][1]
+    sigmaf <- dsigma$x[dsigma$y == max(dsigma$y)][1]
+      th <- th/sigmaf
+      y <- y/sigmaf 
+      sigma <- sigma*sqrt(sigmaf)
 
     if (sequence) sigmas[i] <- sigma
 
@@ -137,17 +286,12 @@ awssigmc <- function(y,                 # data
   ## END PS iteration
 
   ## estimate parameter
-  eta <- sqrt(pmax( 0, th/sigma^2-2*ncoils)) 
+  eta <- sqrt(pmax( 0, th-2*ncoils)) 
   dim(eta) <- ddim
 
   ## this is the result
   invisible(list(sigma = if(sequence) sigmas else sigma,
-                 theta = eta*sigma, 
-                 ni = ni,
-                 ind = ind,
-                 s2 = s2,
-                 itrunc = indt,
-                 cw = cw))
+                 theta = eta*sigma))
 }
 
 
@@ -386,17 +530,17 @@ awsgsigma2 <- function(y,mask,hobj,tobj,varprop,h0,thmin,thmax){
     theta <- tobj$theta[ind]
     wght <- (tobj$gi[ind]^2-tobj$gi2[ind])/tobj$gi[ind]^2
     coef <- coefficients(lm(absresid~theta,weights=wght^2))
-  gamma <- pmin(tobj$gi/hobj$bi,1)
-  theta <- gamma*tobj$theta+(1-gamma)*hobj$theta
 # force positive variance for positive mean by increasing variance estimate
-  if(coef[2] <0){
-     coef[1] <- coef[1]+(thmax+thmin)/2*coef[2]
+  if(coef[2] < 0){
+     coef[1] <- coefficients(lm(absresid~1,weights=wght^2))
      coef[2] <- 0
   }
   if(coef[1] <0.5){
-     coef[2] <- coef[2]+coef[1]/thmax
-     coef[1] <- 0.5
+     coef[2] <- coefficients(lm(absresid~theta-1,weights=wght^2))
+     coef[1] <- 0
   }
+  gamma <- pmin(tobj$gi/hobj$bi,1)
+  theta <- gamma*tobj$theta+(1-gamma)*hobj$theta
   sigma2 <- (coef[1]+coef[2]*theta)^2
   varquantile <- varprop*mean(sigma2[mask])
   sigma2 <- pmax(sigma2,varquantile)
